@@ -1,5 +1,5 @@
 package Mail::DMARC::PurePerl;
-our $VERSION = '1.20140622'; # VERSION
+our $VERSION = '1.20200214';
 use strict;
 use warnings;
 
@@ -12,9 +12,14 @@ sub init {
     $self->is_subdomain(0);
     $self->{header_from} = undef;
     $self->{header_from_raw} = undef;
+    $self->{envelope_to} = undef;
+    $self->{envelope_from} = undef;
+    $self->{source_ip} = undef;
     $self->{policy} = undef;
     $self->{result} = undef;
     $self->{report} = undef;
+    $self->{spf} = undef;
+    $self->{dkim} = undef;
     return;
 }
 
@@ -25,9 +30,9 @@ sub validate {
     $self->result->result('fail');        # set a couple
     $self->result->disposition('none');   # defaults
 
-# 11.2.1 Extract RFC5322.From domain
+    # 11.2.1 Extract RFC5322.From domain
     my $from_dom = $self->get_from_dom() or return $self->result;
-# 9.6. reject email if the domain appears to not exist
+    # 9.6. reject email if the domain appears to not exist
     $self->exists_in_dns() or return $self->result;
     $policy ||= $self->discover_policy();  # 11.2.2 Query DNS for DMARC policy
     $policy or return $self->result;
@@ -36,10 +41,15 @@ sub validate {
     #         specifying that a pass from one authentication test allows one
     #         to skip the other(s). All are required for reporting.
 
-    $self->is_dkim_aligned;   # 11.2.3. DKIM signature verification checks
-    $self->is_spf_aligned;    # 11.2.4. SPF validation checks
-    $self->is_aligned()       # 11.2.5. identifier alignment checks
-        and return $self->result;
+    eval { $self->is_dkim_aligned; };  # 11.2.3. DKIM signature verification checks
+    eval { $self->is_spf_aligned;  };  # 11.2.4. SPF validation checks
+    my $aligned = $self->is_aligned(); # 11.2.5. identifier alignment checks
+
+    if ($self->config->{report_store}{auto_save}) {
+        eval { $self->save_aggregate(); };
+    }
+
+    return $self->result if $aligned;
 
     my $effective_p
         = $self->is_subdomain && defined $policy->sp
@@ -65,11 +75,13 @@ sub validate {
 
     # The stated percentage of messages that fail the DMARC test MUST be
     # subjected to whatever policy is selected by the "p" or "sp" tag
-    if ( int( rand(100) ) >= $policy->pct ) {
-        $self->result->reason( type => 'sampled_out' );
+    if ( int( rand(100) ) < $policy->pct ) {
+        $self->result->disposition($effective_p);
         return $self->result;
     }
 
+    $self->result->reason( type => 'sampled_out' );
+
     # Those that are not thus selected MUST instead be subjected to the next
     # policy lower in terms of severity.  In decreasing order of severity,
     # the policies are "reject", "quarantine", and "none".
@@ -78,6 +90,26 @@ sub validate {
     return $self->result;
 }
 
+sub save_aggregate {
+    my ( $self ) = @_;
+
+    my $pol;
+    eval { $pol = $self->result->published; };
+    if ( $pol && $self->has_valid_reporting_uri($pol->rua) ) {
+        my @valid_report_uris = $self->get_valid_reporting_uri($pol->rua);
+
+        my $filtered_report_uris = join( ',',
+            map { $_->{'uri'} . ( ( $_->{'max_bytes'} > 0 ) ? ( '!' . $_->{'max_bytes'} ) : q{} ) }
+                @valid_report_uris
+        );
+
+        $self->result->published->rua( $filtered_report_uris );
+
+        return $self->SUPER::save_aggregate();
+    }
+    return;
+}
+
 sub discover_policy {
     my $self     = shift;
     my $from_dom = shift || $self->header_from or croak;
@@ -85,8 +117,9 @@ sub discover_policy {
     my $org_dom  = $self->get_organizational_domain($from_dom);
 
     # 9.1  Mail Receivers MUST query the DNS for a DMARC TXT record
-    my $matches = $self->fetch_dmarc_record( $from_dom, $org_dom );
+    my ($matches, $at_dom) = $self->fetch_dmarc_record( $from_dom, $org_dom );
     if (0 == scalar @$matches ) {
+        $self->result->result('none');
         $self->result->reason( type => 'other', comment => 'no policy' );
         return;
     };
@@ -100,12 +133,13 @@ sub discover_policy {
     }
 
     my $policy;
-    eval { $policy = $self->policy( $matches->[0] ) } or return;
+    if (!$at_dom) { $at_dom = $from_dom; }
+    my $policy_str = "domain=$at_dom;" . $matches->[0];  # prefix with domain
+    eval { $policy = $self->policy( $policy_str ) } or return;
     if ($@) {
         $self->result->reason( type => 'other', comment => "policy parse error: $@" );
         return;
     };
-    $policy->domain($from_dom);
     $self->result->published($policy);
 
     # 9.6 If a retrieved policy record does not contain a valid "p" tag, or
@@ -171,10 +205,7 @@ sub is_dkim_aligned {
 
     # Required in report: DKIM-Domain, DKIM-Identity, DKIM-Selector
     foreach my $dkim_ref ( $self->get_dkim_pass_sigs() ) {
-        my $dkim_dom = $dkim_ref->{domain};
-
-        # 4.3.1 make sure $dkim_dom is not a public suffix
-        next if $self->is_public_suffix($dkim_dom);
+        my $dkim_dom = lc $dkim_ref->{domain};
 
         my $dkmeta = {
             domain   => $dkim_ref->{domain},
@@ -213,7 +244,7 @@ sub is_spf_aligned {
 
     if ( !$spf_dom && !$self->spf ) { croak "missing SPF!"; }
     if ( !$spf_dom ) {
-        my @passes = grep { $_->{result} =~ /pass/i } @{ $self->spf };
+        my @passes = grep { $_->{result} && $_->{result} =~ /pass/i } @{ $self->spf };
         if (scalar @passes == 0) {
             $self->result->spf('fail');
             return 0;
@@ -259,6 +290,7 @@ sub is_spf_aligned {
 sub is_whitelisted {
     my $self = shift;
     my $s_ip = shift || $self->source_ip;
+    return if ! defined $s_ip;
     if ( ! $self->{_whitelist} ) {
         my $white_file = $self->config->{smtp}{whitelist} or return;
         return if ! -f $white_file || ! -r $white_file;
@@ -277,10 +309,17 @@ sub is_whitelisted {
             ($comment && $comment =~ /\S/ ? ('comment' => $comment) : () ),
             );
     return $type;
-};
+}
 
 sub has_valid_reporting_uri {
     my ( $self, $rua ) = @_;
+    my @valid_reporting_uris = $self->get_valid_reporting_uri( $rua );
+    return scalar @valid_reporting_uris;
+}
+
+sub get_valid_reporting_uri {
+    my ( $self, $rua ) = @_;
+    return unless $rua;
     my $recips_ref = $self->report->uri->parse($rua);
     my @has_permission;
     foreach my $uri_ref (@$recips_ref) {
@@ -289,16 +328,16 @@ sub has_valid_reporting_uri {
             next;
         }
         my $ext = $self->verify_external_reporting($uri_ref);
-        push @has_permission, $ext if $ext;
+        push @has_permission, $uri_ref if $ext;
     }
-    return @has_permission if wantarray;
-    return scalar @has_permission;
+    return @has_permission;
 }
 
 sub get_dkim_pass_sigs {
     my $self = shift;
 
     my $dkim_sigs = $self->dkim or return ();    # message not signed
+
     if ( 'ARRAY' ne ref $dkim_sigs ) {
         croak "dkim needs to be an array reference!";
     }
@@ -318,7 +357,7 @@ sub get_organizational_domain {
     #     labels.  Number these labels from right-to-left; e.g. for
     #     "example.com", "com" would be label 1 and "example" would be
     #     label 2.;
-    my @labels = reverse split /\./, $from_dom;
+    my @labels = reverse split /\./, lc $from_dom;
 
     # 4.3 Search the public suffix list for the name that matches the
     #     largest number of labels found in the subject DNS domain.  Let
@@ -350,8 +389,11 @@ sub exists_in_dns {
     my $self = shift;
     my $from_dom = shift || $self->header_from or croak "no header_from!";
 
-  # 9.6 # If the RFC5322.From domain does not exist in the DNS, Mail Receivers
-  #     SHOULD direct the receiving SMTP server to reject the message {R9}.
+  # rfc7489 6.6.3
+  #     If the set produced by the mechanism above contains no DMARC policy
+  #     record (i.e., any indication that there is no such record as opposed
+  #     to a transient DNS error), Mail Receivers SHOULD NOT apply the DMARC
+  #     mechanism to the message.
 
     my $org_dom = $self->get_organizational_domain($from_dom);
     my @todo    = $from_dom;
@@ -368,7 +410,8 @@ sub exists_in_dns {
         $matched++ and next if $self->has_dns_rr( 'AAAA', $_ );
     }
     if ( !$matched ) {
-        $self->result->disposition('reject');
+        $self->result->result('none');
+        $self->result->disposition('none');
         $self->result->reason(
             type    => 'other',
             comment => "$from_dom not in DNS"
@@ -386,7 +429,7 @@ sub fetch_dmarc_record {
     $self->is_subdomain( defined $org_dom ? 0 : 1 );
     my @matches = ();
     my $query = $self->get_resolver->send( "_dmarc.$zone", 'TXT' )
-        or return \@matches;
+        or return (\@matches, $zone);
     for my $rr ( $query->answer ) {
         next if $rr->type ne 'TXT';
 
@@ -396,20 +439,22 @@ sub fetch_dmarc_record {
         print "\n" . $rr->txtdata . "\n\n" if $self->verbose;
         push @matches, join( '', $rr->txtdata );    # join long records
     }
-    return \@matches if scalar @matches;            # found one! (at least)
+    if (scalar @matches) {
+        return \@matches, $zone;            # found one! (at least)
+    }
 
     #   3.  If the set is now empty, the Mail Receiver MUST query the DNS for
     #       a DMARC TXT record at the DNS domain matching the Organizational
     #       Domain in place of the RFC5322.From domain in the message (if
     #       different).  This record can contain policy to be asserted for
     #       subdomains of the Organizational Domain.
-    if ( defined $org_dom ) {    #  <- recursion break
+    if ( defined $org_dom ) {                              #  <- recursion break
         if ( $org_dom ne $zone ) {
             return $self->fetch_dmarc_record($org_dom);    #  <- recursion
         }
     }
 
-    return \@matches;
+    return \@matches, $zone;
 }
 
 sub get_from_dom {
@@ -421,9 +466,9 @@ sub get_from_dom {
         return;
     };
 
-# TODO: the From header can contain multiple addresses and should be
-# parsed as described in RFC 2822. If From has multiple-addresses,
-# then parse and use the domain in the Sender header.
+    # TODO: the From header can contain multiple addresses and should be
+    # parsed as described in RFC 2822. If From has multiple-addresses,
+    # then parse and use the domain in the Sender header.
 
     # This returns only the domain in the last email address.
     # Caller can pass in pre-parsed from_dom if this doesn't suit them.
@@ -431,7 +476,7 @@ sub get_from_dom {
     # I care only about the domain. This is way faster than RFC2822 parsing
 
     my ($from_dom) = ( split /@/, $header )[-1]; # grab everything after the @
-    ($from_dom) = split /(\s+|>)/, $from_dom;    # remove trailing cruft
+    ($from_dom) = split /(\s+|>)/, lc $from_dom; # remove trailing cruft
     if ( !$from_dom ) {
         $self->result->reason(
             type    => 'other',
@@ -448,7 +493,7 @@ sub external_report {
         or croak "published policy not tagged!";
 
     if ( 'mailto' eq $uri->scheme ) {
-        my $dest_email = $uri->path;
+        my $dest_email = lc $uri->path;
         my ($dest_host) = ( split /@/, $dest_email )[-1];
         if ( $self->get_organizational_domain( $dest_host )
                 eq
@@ -533,7 +578,7 @@ sub verify_external_reporting {
 
 1;
 
-# ABSTRACT: Pure Perl implementation of DMARC
+__END__
 
 =pod
 
@@ -543,7 +588,7 @@ Mail::DMARC::PurePerl - Pure Perl implementation of DMARC
 
 =head1 VERSION
 
-version 1.20140622
+version 1.20200214
 
 =head1 METHODS
 
@@ -700,17 +745,17 @@ Matt Simerson <msimerson@cpan.org>
 
 Davide Migliavacca <shari@cpan.org>
 
+=item *
+
+Marc Bradshaw <marc@marcbradshaw.net>
+
 =back
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2014 by ColocateUSA.com.
+This software is copyright (c) 2020 by Matt Simerson.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
 
 =cut
-
-__END__
-sub {}
-
